@@ -32,7 +32,7 @@
 char usage[] =
 
 "\n"
-"  802ether 1.4 - (C) 2004 Christophe Devine\n"
+"  802ether 2.0 - (C) 2004 Christophe Devine\n"
 "\n"
 "  usage: %s <pcap infile> <pcap outfile> [wep key]\n"
 "\n";
@@ -52,7 +52,7 @@ int main( int argc, char *argv[] )
     struct pcap_file_header pfh_out;
     struct pcap_pkthdr pkh;
 
-    int weplen, infile, outfile;
+    int weplen, fd_in, fd_out;
     int n, i, j, p1, p2;
     long int cnt1, cnt2;
     time_t tm_prev;
@@ -77,9 +77,9 @@ int main( int argc, char *argv[] )
             if( n < 0 || n > 255 )
                 goto usage;
 
-            wepkey[weplen] = n;
+            wepkey[weplen++] = n;
 
-            if( ++weplen >= 13 ) break;
+            if( weplen >= 13 ) break;
 
             if( ! ( s = strchr( s, ':' ) ) )
                 break;
@@ -93,17 +93,17 @@ int main( int argc, char *argv[] )
 
     /* open the input file and check the pcap header */
 
-    infile = open( argv[1], O_RDONLY | O_LARGEFILE );
+    fd_in = open( argv[1], O_RDONLY | O_LARGEFILE );
 
-    if( infile < 0 )
+    if( fd_in < 0 )
     {
-        perror( "open(pcap infile)" );
+        perror( "open(pcap fd_in)" );
         return( 1 );
     }
 
     n = sizeof( struct pcap_file_header );
 
-    if( safe_read( infile, &pfh_in, n ) != n )
+    if( safe_read( fd_in, &pfh_in, n ) != n )
     {
         perror( "read(pcap header)" );
         return( 1 );
@@ -120,7 +120,7 @@ int main( int argc, char *argv[] )
     if( pfh_in.linktype != LINKTYPE_IEEE802_11 &&
         pfh_in.linktype != LINKTYPE_PRISM_HEADER )
     {
-        fprintf( stderr, "unsupported linktype %d from pcap infile\n",
+        fprintf( stderr, "unsupported linktype %d from pcap fd_in\n",
                  pfh_in.linktype );
         return( 1 );
     }
@@ -129,12 +129,12 @@ int main( int argc, char *argv[] )
 
     n = sizeof( struct pcap_file_header );
 
-    if( ( outfile = open( argv[2], O_LARGEFILE | O_RDWR ) ) < 0 )
+    if( ( fd_out = open( argv[2], O_LARGEFILE | O_RDWR ) ) < 0 )
     {
-        if( ( outfile = open( argv[2], O_LARGEFILE | O_RDWR |
+        if( ( fd_out = open( argv[2], O_LARGEFILE | O_RDWR |
                                        O_CREAT, 0644 ) ) < 0 )
         {
-            perror( "creat(pcap outfile)" );
+            perror( "creat(pcap fd_out)" );
             return( 1 );
         }
 
@@ -148,7 +148,7 @@ int main( int argc, char *argv[] )
         pfh_out.snaplen         = 65535;
         pfh_out.linktype        = LINKTYPE_ETHERNET;
 
-        if( safe_write( outfile, &pfh_out, n ) != n )
+        if( safe_write( fd_out, &pfh_out, n ) != n )
         {
             perror( "write(pcap header)" );
             return( 1 );
@@ -158,7 +158,7 @@ int main( int argc, char *argv[] )
     {
         /* existing file, check the pcap header contents */
 
-        if( safe_read( outfile, &pfh_out, n ) != n )
+        if( safe_read( fd_out, &pfh_out, n ) != n )
         {
             perror( "read(pcap header)" );
             return( 1 );
@@ -180,7 +180,7 @@ int main( int argc, char *argv[] )
             return( 1 );
         }
 
-        if( lseek64( outfile, 0, SEEK_END ) < 0 )
+        if( lseek64( fd_out, 0, SEEK_END ) < 0 )
         {
             perror( "lseek64(SEEK_END)" );
             return( 1 );
@@ -209,12 +209,12 @@ int main( int argc, char *argv[] )
 
         n = sizeof( pkh );
 
-        if( safe_read( infile, &pkh, n ) != n )
+        if( safe_read( fd_in, &pkh, n ) != n )
             break;
 
-        n = pkh.len;
+        n = pkh.caplen;
 
-        if( safe_read( infile, buffer, n ) != n )
+        if( safe_read( fd_in, buffer, n ) != n )
             break;
 
         cnt1++;
@@ -227,13 +227,13 @@ int main( int argc, char *argv[] )
 
             n = *(int *)( h80211 + 4 );
 
-            if( n < 8 || n >= (int) pkh.len )
+            if( n < 8 || n >= (int) pkh.caplen )
                 continue;
 
-            h80211 += n; pkh.len -= n;
+            h80211 += n; pkh.caplen -= n;
         }
 
-        if( pkh.len < 40 ) continue;
+        if( pkh.caplen < 40 ) continue;
 
         /* only consider data packets */
 
@@ -258,7 +258,7 @@ int main( int argc, char *argv[] )
 
             p1 = p2 = 0;
 
-            for( i = 1, j = 0; i <= (int) pkh.len - 32; i++ )
+            for( i = 1, j = 0; i <= (int) pkh.caplen - 32; i++ )
             {
                 p1 = ( p1 + 1     ) & 0xFF;
                 p2 = ( p2 + S[p1] ) & 0xFF;
@@ -271,7 +271,7 @@ int main( int argc, char *argv[] )
             pkh.len    -= 4 + ( h80211[27] & 0x3F ) + 4;
             pkh.caplen -= 4 + ( h80211[27] & 0x3F ) + 4;
 
-            memcpy( h80211 + 24, h80211 + 28, pkh.len - 24 );
+            memcpy( h80211 + 24, h80211 + 28, pkh.caplen - 24 );
         }
 
         /* remove the 802.11 & LLC header */
@@ -307,16 +307,25 @@ int main( int argc, char *argv[] )
             default:    /* To DS = 1, From DS = 1: RA, TA, DA, SA */
 
                 memcpy( arphdr + 0, h80211 + 16, 6 );
-                memcpy( arphdr + 6, h80211 + 22, 6 );
+                memcpy( arphdr + 6, h80211 + 24, 6 );
                 break;
         }
 
         memcpy( h80211 + 18, arphdr, 12 );
 
-        if( safe_write( outfile, &pkh,  sizeof( pkh ) ) != sizeof( pkh ) ||
-            safe_write( outfile, h80211 + 18, pkh.len ) != (int) pkh.len )
+        n = sizeof( pkh );
+
+        if( safe_write( fd_out, &pkh, n ) != n )
         {
-            perror( "\nwrite" );
+            perror( "\nwrite(packet header)" );
+            return( 1 );
+        }
+
+        n = pkh.caplen;
+
+        if( safe_write( fd_out, h80211 + 18, n ) != n )
+        {
+            perror( "\nwrite(packet data)" );
             return( 1 );
         }
 
